!-----------------------BEGIN NOTICE -- DO NOT EDIT-----------------------
! NASA Goddard Space Flight Center Land Information System (LIS) v7.1
!
! Copyright (c) 2015 United States Government as represented by the
! Administrator of the National Aeronautics and Space Administration.
! All Rights Reserved.
!-------------------------END NOTICE -- DO NOT EDIT-----------------------
#include "LIS_misc.h"
module LIS_RTMoptue_pluginMod
!BOP
!
! !MODULE: LIS_RTMoptue_pluginMod
! 
! !DESCRIPTION: 
!   This module contains the definition of the functions that 
!   need to be defined to enable the use of a land surface 
!   model in parameter estimation setup. 
!   
! !REVISION HISTORY: 
!  16 Jul 09    Sujay Kumar  Initial Specification
! 
  implicit none
  
  PRIVATE
!------------------------------------------------------------------------------
! !PUBLIC MEMBER FUNCTIONS:
!------------------------------------------------------------------------------
  PUBLIC :: LIS_RTMoptue_plugin  
!EOP  

contains
!BOP
! !ROUTINE: LIS_RTMoptue_plugin
!  \label{LIS_RTMoptue_plugin}
!
! !DESCRIPTION:
!
! This is a custom-defined plugin point for introducing a new RTM
! in a parameter estimation mode. The interface mandates that 
! a number of routines be implemented and registered for 
! each of the RTM that is used in a parameter estimation mode. 
!  
! !INTERFACE:
  subroutine LIS_RTMoptue_plugin
!EOP
    use LIS_pluginIndices
#if (defined RTMS)
    use Empty_obsMod, only: Empty_getpeobspred, Empty_setupobspred  ! used for MCSIM as does not rely on obs
    use crtm2em_peMod, only: crtm2em_setup_pedecvars
    use cmem3_peMod, only: cmem3_setup_pedecvars
    use LIS_coreMod,   only : LIS_rc
!    use CRTM2_EMMod
!    use CMEM3_Mod
!!$    use noah271_peMod, only : noah271_setup_pedecvars
!!$    use noah32_peMod, only : noah32_setup_pedecvars

    external CRTM2EM_qcdec
    external CRTM2EM_set_pedecvars
    external CRTM2EM_param_reset
#if 0
    external CRTM2EM_setupobspred_CNRS
    external CRTM2EM_setupobspred_CNRS_MPDI
    external CRTM2EM_getpeobspred_CNRS
    external CRTM2EM_getpeobspred_CNRS_MPDI
#endif
    external CRTM2EM_setupobspred_AMSRE_SR
    external CRTM2EM_getpeobspred_AMSRE_SR

    external CMEM3_qcdec
    external CMEM3_set_pedecvars
    external CMEM3_param_reset
#if 0
    external CMEM3_setupobspred_CNRS
    external CMEM3_getpeobspred_CNRS
    external CMEM3_getpeobspred_CNRS_MPDI
#endif
    external CMEM3_setupobspred_AMSRE_SR
    external CMEM3_getpeobspred_AMSRE_SR

    call registerrtmpesetupdecisionspace(trim(LIS_CRTM2EMId)//char(0),crtm2em_setup_pedecvars)
    call registerrtmpesetdecisionspace(trim(LIS_CRTM2EMId)//char(0), crtm2em_set_pedecvars)

#if 0
    call registerrtmpesetupobspred(trim(LIS_CRTM2EMId)//"+"//trim(LIS_CNRSObsId)//char(0), CRTM2EM_setupobspred_CNRS)
    call registerrtmpesetupobspred(trim(LIS_CRTM2EMId)//"+"//trim(LIS_CNRS_MPDIObsId)//char(0), CRTM2EM_setupobspred_CNRS) !note same call as non-mpdi
#endif
    call registerrtmpesetupobspred(trim(LIS_CRTM2EMId)//"+"//trim(LIS_AMSRE_SRObsId)//char(0), CRTM2EM_setupobspred_AMSRE_SR)
    call registerrtmpesetupobspred(trim(LIS_CRTM2EMId)//"+"//trim(LIS_EmptyObsId)//char(0), Empty_setupobspred)

#if 0
    call registerrtmpegetobspred(trim(LIS_CRTM2EMId)//"+"//trim(LIS_CNRSObsId)//char(0), CRTM2EM_getpeobspred_CNRS)
    call registerrtmpegetobspred(trim(LIS_CRTM2EMId)//"+"//trim(LIS_CNRS_MPDIObsId)//char(0), CRTM2EM_getpeobspred_CNRS_MPDI)
#endif
    call registerrtmpegetobspred(trim(LIS_CRTM2EMId)//"+"//trim(LIS_AMSRE_SRObsId)//char(0), CRTM2EM_getpeobspred_AMSRE_SR)
    call registerrtmpegetobspred(trim(LIS_CRTM2EMId)//"+"//trim(LIS_EmptyObsId)//char(0), Empty_getpeobspred)


    call registerrtmpesetupdecisionspace(trim(LIS_CMEM3Id)//char(0), cmem3_setup_pedecvars)
    call registerrtmpesetdecisionspace(trim(LIS_CMEM3Id)//char(0), cmem3_set_pedecvars)

#if 0
    call registerrtmpesetupobspred(trim(LIS_CMEM3Id)//"+"//trim(LIS_CNRSObsId)//char(0), CMEM3_setupobspred_CNRS)
    call registerrtmpesetupobspred(trim(LIS_CMEM3Id)//"+"//trim(LIS_CNRS_MPDIObsId)//char(0), CMEM3_setupobspred_CNRS) !note same call as non-mpdi
#endif
    call registerrtmpesetupobspred(trim(LIS_CMEM3Id)//"+"//trim(LIS_AMSRE_SRObsId)//char(0), CMEM3_setupobspred_AMSRE_SR)
    call registerrtmpesetupobspred(trim(LIS_CMEM3Id)//"+"//trim(LIS_EmptyObsId)//char(0), Empty_setupobspred)

#if 0
    call registerrtmpegetobspred(trim(LIS_CMEM3Id)//"+"//trim(LIS_CNRSObsId)//char(0), CMEM3_getpeobspred_CNRS)
    call registerrtmpegetobspred(trim(LIS_CMEM3Id)//"+"//trim(LIS_CNRS_MPDIObsId)//char(0), CMEM3_getpeobspred_CNRS_MPDI)
#endif
    call registerrtmpegetobspred(trim(LIS_CMEM3Id)//"+"//trim(LIS_AMSRE_SRObsId)//char(0), CMEM3_getpeobspred_AMSRE_SR)
    call registerrtmpegetobspred(trim(LIS_CMEM3Id)//"+"//trim(LIS_EmptyObsId)//char(0), Empty_getpeobspred)

#endif
  end subroutine LIS_RTMoptue_plugin
end module LIS_RTMoptue_pluginMod
