!-----------------------BEGIN NOTICE -- DO NOT EDIT-----------------------
! NASA Goddard Space Flight Center Land Information System (LIS) v7.1
!
! Copyright (c) 2015 United States Government as represented by the
! Administrator of the National Aeronautics and Space Administration.
! All Rights Reserved.
!-------------------------END NOTICE -- DO NOT EDIT-----------------------
!BOP
! !ROUTINE: clsmf25_setsnowvars
! \label{clsmf25_setsnowvars}
!
! !REVISION HISTORY:
! 27Feb2005: Sujay Kumar; Initial Specification
! 25Jun2006: Sujay Kumar: Updated for the ESMF design
!  02 Mar 2010: Sujay Kumar; Modified for Noah 3.1
!
! !INTERFACE:
subroutine clsmf25_setsnowvars(n, LSM_State)
! !USES:
  use ESMF
  use LIS_coreMod
  use clsmf25_lsmMod
  use clsmf25_constants
  use LIS_logMod, only : LIS_logunit, LIS_verify

  implicit none
! !ARGUMENTS: 
  integer, intent(in)    :: n
  type(ESMF_State)       :: LSM_State
! 
! !DESCRIPTION:
! 
!  This routine assigns the snow progognostic variables to noah's
!  model space. 
! 
!EOP
!  real, parameter        :: WEMIN  = 13.0   ! [KG/M2]
  type(ESMF_Field)       :: wesn1Field,wesn2Field,wesn3Field

  integer                :: t
  integer                :: status
  real, pointer          :: wesn1(:),wesn2(:),wesn3(:)
  real                   :: wesn_minus(3), sndz_minus(3), htsn_minus(3)
  real, dimension(3,2)   :: ds
  real, dimension(4)     :: sdold, sdnew
  real, dimension(4,2)   :: h,s
  integer                :: i,j,isnow
  real                   :: dz, wesnsum
  real                   :: snow_density,areasc_minus,areasc_plus
  real                   :: check_val 
  real                   :: wesn_ratio(3)
!  integer, parameter ::  N_snow = 3
!  real, parameter :: dz1max = 0.05   ! [m]
  real, parameter :: small  = 1.e-20
  real                   :: swediff
  real                   :: swe_pchange, sndz_pchange

  call ESMF_StateGet(LSM_State,"Water Equivalent Snow 1",wesn1Field,rc=status)
  call LIS_verify(status)
  call ESMF_StateGet(LSM_State,"Water Equivalent Snow 2",wesn2Field,rc=status)
  call LIS_verify(status)
  call ESMF_StateGet(LSM_State,"Water Equivalent Snow 3",wesn3Field,rc=status)
  call LIS_verify(status)

  call ESMF_FieldGet(wesn1Field,localDE=0,farrayPtr=wesn1,rc=status)
  call LIS_verify(status)
  call ESMF_FieldGet(wesn2Field,localDE=0,farrayPtr=wesn2,rc=status)
  call LIS_verify(status)
  call ESMF_FieldGet(wesn3Field,localDE=0,farrayPtr=wesn3,rc=status)
  call LIS_verify(status)

  do t=1,LIS_rc%npatch(n,LIS_rc%lsm_index)

     wesn_minus(:) = clsmf25_struc(n)%cat_progn(t)%wesn(:)
     sndz_minus(:) = clsmf25_struc(n)%cat_progn(t)%sndz(:)
     htsn_minus(:) = clsmf25_struc(n)%cat_progn(t)%htsn(:)
     areasc_minus = min(1.0,(clsmf25_struc(n)%cat_progn(t)%wesn(1)+&
          clsmf25_struc(n)%cat_progn(t)%wesn(2)+&
          clsmf25_struc(n)%cat_progn(t)%wesn(3))/wemin)

     if(sum(clsmf25_struc(n)%cat_progn(t)%sndz(:)).gt.0) then 
        snow_density = max(sum(clsmf25_struc(n)%cat_progn(t)%wesn(:))/&
             (sum(clsmf25_struc(n)%cat_progn(t)%sndz(:))*areasc_minus), rhofs)
     else
        snow_density = 0.0
     endif

     if(clsmf25_struc(n)%cat_progn(t)%wesn(1).gt.0) then 
        wesn_ratio(1) = wesn1(t)/clsmf25_struc(n)%cat_progn(t)%wesn(1)
     else
        wesn_ratio(1) = 0.0
     endif
     if(clsmf25_struc(n)%cat_progn(t)%wesn(2).gt.0) then 
        wesn_ratio(2) = wesn2(t)/clsmf25_struc(n)%cat_progn(t)%wesn(2)
     else
        wesn_ratio(2) = 0.0
     endif
     if(clsmf25_struc(n)%cat_progn(t)%wesn(3).gt.0) then 
        wesn_ratio(3) = wesn3(t)/clsmf25_struc(n)%cat_progn(t)%wesn(3)
     else
        wesn_ratio(3) = 0.0
     endif
     do isnow=1,3
        clsmf25_struc(n)%cat_progn(t)%htsn(isnow) = &
             clsmf25_struc(n)%cat_progn(t)%htsn(isnow) * wesn_ratio(isnow)
     enddo
     
     areasc_plus = min(1.0,((wesn1(t)+wesn2(t)+wesn3(t))/wemin))
     
     clsmf25_struc(n)%cat_progn(t)%wesn(1) = wesn1(t)
     clsmf25_struc(n)%cat_progn(t)%wesn(2) = wesn2(t)
     clsmf25_struc(n)%cat_progn(t)%wesn(3) = wesn3(t)

     do isnow = 1, 3

        if(wesn_ratio(isnow).gt.0.0) then 
           ! CASE 1, SWE^+, SWE^- >= Wo , or fSCA+, fSCA- =1
#if 0 
           if ( (areasc_minus.eq.1.0) .and. (areasc_plus.eq.1.0) ) then
              clsmf25_struc(n)%cat_progn(t)%sndz(isnow) = &
                   clsmf25_struc(n)%cat_progn(t)%sndz(isnow) * wesn_ratio(isnow)
           endif
           ! CASE 2, SWE^+, SWE^- < Wo , or fSCA+, fSCA- <1
           if ( (areasc_minus.lt.1.0) .and. (areasc_plus.lt.1.0) ) then

!              clsmf25_struc(n)%cat_progn(t)%sndz(isnow) = &
!                   clsmf25_struc(n)%cat_progn(t)%sndz(isnow) 
#endif
              if(isnow.eq.1) then 
                 if(snow_density.gt.0) then 
                    clsmf25_struc(n)%cat_progn(t)%sndz(isnow) =  &
                         wesn1(t)/&
                         (snow_density*areasc_plus)
                 endif
              endif
              if(isnow.eq.2) then 
                 if(snow_density.gt.0) then 
                    clsmf25_struc(n)%cat_progn(t)%sndz(isnow) =  &
                         wesn2(t)/&
                         (snow_density*areasc_plus)
                 endif
              endif
              if(isnow.eq.3) then 
                 if(snow_density.gt.0) then 
                    clsmf25_struc(n)%cat_progn(t)%sndz(isnow) =  &
                         wesn3(t)/&
                         (snow_density*areasc_plus)
                 endif
              endif
#if 0 
           endif

           ! CASE 3, SWE^+ > Wo ; SWE^- < Wo , or fSCA+ = 1.0 ; fSCA- <1
           if ( (areasc_minus.lt.1.0) .and. (areasc_plus.eq.1.0) ) then
              clsmf25_struc(n)%cat_progn(t)%sndz(isnow) = &
                   clsmf25_struc(n)%cat_progn(t)%sndz(isnow) * &
                   clsmf25_struc(n)%cat_progn(t)%wesn(isnow) / WEMIN
           endif
           ! CASE 4, SWE^+ < Wo ; SWE^- > Wo , or fSCA+ < 1.0 ; fSCA- =1
           if ( (areasc_minus.eq.1.0) .and. (areasc_plus.lt.1.0) ) then
              clsmf25_struc(n)%cat_progn(t)%sndz(isnow) = &
                   clsmf25_struc(n)%cat_progn(t)%sndz(isnow) * &
                   WEMIN / (clsmf25_struc(n)%cat_progn(t)%wesn(isnow) / &
                   wesn_ratio(isnow)) 
           endif
#endif           
        endif
     enddo

!     if(t.eq.1401.and.LIS_localPet.eq.140) then 
!        print*, 'before relayer '
!        print*, clsmf25_struc(n)%cat_progn(t)%wesn
!        print*, clsmf25_struc(n)%cat_progn(t)%sndz
!     endif

     call clsmf25_relayer_snow(t, clsmf25_struc(n)%cat_progn(t)%htsn(1:3), &
          clsmf25_struc(n)%cat_progn(t)%wesn(1:3), &
          clsmf25_struc(n)%cat_progn(t)%sndz(1:3))

     if((sum(clsmf25_struc(n)%cat_progn(t)%wesn(:)).eq.0.and.&
          sum(clsmf25_struc(n)%cat_progn(t)%sndz(:)).ne.0).or.&
          sum(clsmf25_struc(n)%cat_progn(t)%wesn(:)).ne.0.and.&
          sum(clsmf25_struc(n)%cat_progn(t)%sndz(:)).eq.0) then 
        
        clsmf25_struc(n)%cat_progn(t)%wesn = 0.0
        clsmf25_struc(n)%cat_progn(t)%sndz = 0.0
!        print*, 'problem', t, LIS_localPet
!        print*, wesn_minus
!        print*, sndz_minus
!        print*, clsmf25_struc(n)%cat_progn(t)%wesn
!        print*, clsmf25_struc(n)%cat_progn(t)%sndz
!        stop
     endif

     swe_pchange = 0.0
     sndz_pchange = 0.0

     if(sum(wesn_minus).gt.0) then 
        swe_pchange = abs((sum(clsmf25_struc(n)%cat_progn(t)%wesn(:)) - & 
             sum(wesn_minus))*100.0/sum(wesn_minus))
     endif
     if(sum(sndz_minus).gt.0) then 
        sndz_pchange = abs((sum(clsmf25_struc(n)%cat_progn(t)%sndz(:)) - & 
             sum(sndz_minus))*100.0/sum(sndz_minus))
     endif

!     if(swe_pchange.gt.20.or.sndz_pchange.gt.20) then 
     if((swe_pchange.gt.60.or.sndz_pchange.gt.60).and.&
          (sum(clsmf25_struc(n)%cat_progn(t)%wesn).gt.4*wemin)) then 
!     if(t.eq.1401.and.LIS_localPet.eq.140) then 
        if(sndz_pchange.eq.0) then 
           clsmf25_struc(n)%cat_progn(t)%wesn(:) = wesn_minus(:)
           clsmf25_struc(n)%cat_progn(t)%sndz(:) = sndz_minus(:)
           clsmf25_struc(n)%cat_progn(t)%htsn(:) = htsn_minus(:)
        elseif((swe_pchange/sndz_pchange.lt.0.8) .or. &
             (swe_pchange/sndz_pchange.gt.1.2) ) then 
!ignore the update. 
!           print*, 'problem', t, LIS_localPet
!           print*, 'pchange ',swe_pchange, sndz_pchange
!           print*, 'areasc ',areasc_minus, areasc_plus
!           print*, 'dens ',snow_density, sum(wesn_minus)/(sum(sndz_minus)*areasc_minus)
!           print*, wesn_ratio
!           print*, wesn_minus
!           print*, sndz_minus
!           print*, clsmf25_struc(n)%cat_progn(t)%wesn
!           print*, clsmf25_struc(n)%cat_progn(t)%sndz
           
           clsmf25_struc(n)%cat_progn(t)%wesn(:) = wesn_minus(:)
           clsmf25_struc(n)%cat_progn(t)%sndz(:) = sndz_minus(:)
           clsmf25_struc(n)%cat_progn(t)%htsn(:) = htsn_minus(:)

        endif
     endif

  enddo
end subroutine clsmf25_setsnowvars

#if 0 
  do t=1,LIS_rc%npatch(n,LIS_rc%lsm_index)
!     print*, 'upd ',t, clsmf25_struc(n)%cat_progn(t)%wesn(1), wesn1(t),&
!          clsmf25_struc(n)%cat_progn(t)%wesn(2), wesn2(t),&
!          clsmf25_struc(n)%cat_progn(t)%wesn(3), wesn3(t)

!     if(LIS_localPet.eq.96.and. t.eq.5299) then 
!        print*, 'da0 ',clsmf25_struc(n)%cat_progn(t)%catdef
!     endif
     areasc = min(1.0,(clsmf25_struc(n)%cat_progn(t)%wesn(1)+&
          clsmf25_struc(n)%cat_progn(t)%wesn(2)+&
          clsmf25_struc(n)%cat_progn(t)%wesn(3))/wemin)
     areasc1 = min(1.0,((wesn1(t)+wesn2(t)+wesn3(t))/wemin))
     swediff = (wesn1(t)+wesn2(t)+wesn3(t)-&
          clsmf25_struc(n)%cat_progn(t)%wesn(1)-&
          clsmf25_struc(n)%cat_progn(t)%wesn(2)-&
          clsmf25_struc(n)%cat_progn(t)%wesn(3))
     
     if((wesn1(t) + wesn2(t) + wesn3(t)) > 20.0 .and.& 
          areasc.gt.0 .and. & 
          clsmf25_struc(n)%cat_progn(t)%sndz(1).gt.0.and.&
          clsmf25_struc(n)%cat_progn(t)%sndz(2).gt.0.and.&
          clsmf25_struc(n)%cat_progn(t)%sndz(3).gt.0 .and.&
          abs(swediff) < 500.0 ) then 

        if(wesn1(t).gt.2.0) then
           snow_density = max(clsmf25_struc(n)%cat_progn(t)%wesn(1)/&
                (clsmf25_struc(n)%cat_progn(t)%sndz(1)*areasc), rhofs)
           clsmf25_struc(n)%cat_progn(t)%htsn(1) = &
                clsmf25_struc(n)%cat_progn(t)%htsn(1) &
                * wesn1(t)/clsmf25_struc(n)%cat_progn(t)%wesn(1)
           clsmf25_struc(n)%cat_progn(t)%sndz(1) =  &
                wesn1(t)/&
                (snow_density*areasc1)
           
           clsmf25_struc(n)%cat_progn(t)%wesn(1) = wesn1(t)
           
        else
           check_val = clsmf25_struc(n)%cat_progn(t)%catdef -&
                (wesn1(t) - clsmf25_struc(n)%cat_progn(t)%wesn(1))
           if(check_val.gt.0.1) then 
              clsmf25_struc(n)%cat_progn(t)%catdef &
                   = check_val
           endif
        end if
        
        if(wesn2(t).gt.2.0) then

           snow_density = max(clsmf25_struc(n)%cat_progn(t)%wesn(2)/&
                (clsmf25_struc(n)%cat_progn(t)%sndz(2)*areasc),rhofs)
           clsmf25_struc(n)%cat_progn(t)%htsn(2) = &
                clsmf25_struc(n)%cat_progn(t)%htsn(2) &
                * wesn2(t)/clsmf25_struc(n)%cat_progn(t)%wesn(2)
           clsmf25_struc(n)%cat_progn(t)%sndz(2) =  &
                wesn2(t)/&
                (snow_density*areasc1)
           clsmf25_struc(n)%cat_progn(t)%wesn(2) = wesn2(t)

        else
           check_val = clsmf25_struc(n)%cat_progn(t)%catdef - &
                (wesn2(t)-clsmf25_struc(n)%cat_progn(t)%wesn(2))
           if(check_val.gt.0.1) then 
              clsmf25_struc(n)%cat_progn(t)%catdef &
                   = check_val
           endif
        end if
        
        if(wesn3(t).gt.2.0) then
           snow_density = max(clsmf25_struc(n)%cat_progn(t)%wesn(3)/&
                (clsmf25_struc(n)%cat_progn(t)%sndz(3)*areasc),rhofs)
           clsmf25_struc(n)%cat_progn(t)%htsn(3) = &
                clsmf25_struc(n)%cat_progn(t)%htsn(3) &
                * wesn3(t)/clsmf25_struc(n)%cat_progn(t)%wesn(3)
           clsmf25_struc(n)%cat_progn(t)%sndz(3) = &
                wesn3(t)/&
                (snow_density*areasc1)
           clsmf25_struc(n)%cat_progn(t)%wesn(3) = wesn3(t)
           
        else
           check_val = clsmf25_struc(n)%cat_progn(t)%catdef - &
                (wesn3(t)-clsmf25_struc(n)%cat_progn(t)%wesn(3))
           if(check_val.gt.0.1) then 
              clsmf25_struc(n)%cat_progn(t)%catdef &
                   = check_val
           endif
        end if
        
     ! relayer in order to avoid exeeding max sndz in top snow layer

     !**** Initialize some variables.
        
        h  = 0.
        s  = 0.
        ds = 0.
        dz = 0.
        
        !**** Compute specific heat & water contents of old layers.
        
        do i=1,N_snow
           if (clsmf25_struc(n)%cat_progn(t)%sndz(i) > 0.) then
              h(i,1) = clsmf25_struc(n)%cat_progn(t)%htsn(i)/clsmf25_struc(n)%cat_progn(t)%sndz(i)
              h(i,2) =  clsmf25_struc(n)%cat_progn(t)%wesn(i)/clsmf25_struc(n)%cat_progn(t)%sndz(i)
           endif
        enddo

        !**** Obtain old & new layer thicknesses & boundaries.
        
        sdold = 0.
        sdnew = 0.
        
        do i=N_snow,1,-1
           sdold(i) = sdold(i+1) + clsmf25_struc(n)%cat_progn(t)%sndz(i)
        enddo
        
        clsmf25_struc(n)%cat_progn(t)%sndz = sdold(1)/float(N_snow)
        if(clsmf25_struc(n)%cat_progn(t)%sndz(1) > dz1max) then
           clsmf25_struc(n)%cat_progn(t)%sndz(2:) = (sdold(1)-dz1max)/float(N_snow-1)
           clsmf25_struc(n)%cat_progn(t)%sndz(1)  = dz1max
        endif

        do i=N_snow,1,-1
           sdnew(i) = sdnew(i+1) + clsmf25_struc(n)%cat_progn(t)%sndz(i)
        enddo
        
        !**** Since the snow boundary has moved, redistribute heat
        !     contents & water equivalents of old to new snow layers.
        
        do i=1,N_snow
           
           j = i
           dz=sdnew(i+1)-sdold(i+1)
           if(dz < 0.) j = i + 1
           s(i+1,:) = h(j,:)*dz
           ds(i,:)  = s(i,:) - s(i+1,:)
        enddo
        
        clsmf25_struc(n)%cat_progn(t)%htsn = clsmf25_struc(n)%cat_progn(t)%htsn + ds(:,1)
        clsmf25_struc(n)%cat_progn(t)%wesn  = clsmf25_struc(n)%cat_progn(t)%wesn  + ds(:,2)
        
        wesnsum = (clsmf25_struc(n)%cat_progn(t)%wesn(1)+&
          clsmf25_struc(n)%cat_progn(t)%wesn(2)+&
          clsmf25_struc(n)%cat_progn(t)%wesn(3))

        areasc = min(1.0,(clsmf25_struc(n)%cat_progn(t)%wesn(1)+&
             clsmf25_struc(n)%cat_progn(t)%wesn(2)+&
             clsmf25_struc(n)%cat_progn(t)%wesn(3))/wemin)

        if(wesnsum < wemin) then 
           clsmf25_struc(n)%cat_progn(t)%sndz = clsmf25_struc(n)%cat_progn(t)%sndz / (areasc + small)
        endif
        !**** End relayer
        
     else
        check_val = clsmf25_struc(n)%cat_progn(t)%catdef - &
             ((wesn1(t)- clsmf25_struc(n)%cat_progn(t)%wesn(1)) &
             + (wesn2(t) - clsmf25_struc(n)%cat_progn(t)%wesn(2)) &
             + (wesn3(t) - clsmf25_struc(n)%cat_progn(t)%wesn(3)))
        if(check_val.gt.0.1) then 
           clsmf25_struc(n)%cat_progn(t)%catdef &
                = check_val
        endif        
     end if
  enddo
#endif  


