!-----------------------BEGIN NOTICE -- DO NOT EDIT-----------------------
! NASA Goddard Space Flight Center Land Information System (LIS) v7.1
!
! Copyright (c) 2015 United States Government as represented by the
! Administrator of the National Aeronautics and Space Administration.
! All Rights Reserved.
!-------------------------END NOTICE -- DO NOT EDIT-----------------------
!BOP
! !ROUTINE: noah33_getsmpred
! \label{noah33_getsmpred}
!
! !REVISION HISTORY:
! 27Feb2005: Sujay Kumar; Initial Specification
! 25Jun2006: Sujay Kumar: Updated for the ESMF design
!
! !INTERFACE:
subroutine noah33_getsmpred(n, obs_pred)
! !USES:
  use ESMF
  use LIS_constantsMod
  use LIS_coreMod, only : LIS_rc,LIS_surface
  use LIS_dataAssimMod
  use noah33_lsmMod
!EOP

  implicit none
! !ARGUMENTS: 
  integer, intent(in)    :: n
  real                   :: obs_pred(LIS_rc%ngrid(n),LIS_rc%nensem(n))
!
! !DESCRIPTION:
!
!  Returns the Soil moisture obs pred (model's estimate of 
!  observations) for data assimilation
! 
!  The arguments are: 
!  \begin{description}
!  \item[n] index of the nest \newline
!  \item[obs\_pred] model's estimate of observations \newline
!  \end{description}
!EOP
  integer                :: count1(LIS_rc%ngrid(n),LIS_rc%nensem(n))  
  integer                :: i,t,m,gid
  real                   :: inputs_tp(6), sm_out
  character*50           :: units_tp(6)

  if(LIS_rc%useANNinDA(n).ne.1) then 
     obs_pred = 0.0
     count1 = 0
     do i=1,LIS_rc%npatch(n,LIS_rc%lsm_index),LIS_rc%nensem(n)
        do m=1,LIS_rc%nensem(n)
           t = i+m-1
           gid = LIS_surface(n,LIS_rc%lsm_index)%tile(t)%index        
           obs_pred(gid,m)= obs_pred(gid,m)+ noah33_struc(n)%noah(t)%smc(1)
           count1(gid,m) = count1(gid,m) +1
        enddo
     enddo
     
     do i=1,LIS_rc%ngrid(n)
        do m=1,LIS_rc%nensem(n)
           obs_pred(i,m) = obs_pred(i,m)/(count1(i,m))
        enddo
     enddo
  else
     obs_pred = 0.0
     count1 = 0 
     do i=1,LIS_rc%npatch(n,LIS_rc%lsm_index),LIS_rc%nensem(n)
        do m=1,LIS_rc%nensem(n)
           t = i+m-1
           gid = LIS_surface(n,LIS_rc%lsm_index)%tile(t)%index
           
           inputs_tp(1) = noah33_struc(n)%noah(t)%smc(1)
           inputs_tp(2) = noah33_struc(n)%noah(t)%rainf
           inputs_tp(3) = noah33_struc(n)%noah(t)%snowf
           inputs_tp(4) = noah33_struc(n)%noah(t)%shdfac
           inputs_tp(5) = noah33_struc(n)%noah(t)%stc(1)
           inputs_tp(6) = noah33_struc(n)%noah(t)%sneqv*LIS_CONST_RHOFW

           units_tp(1) = "m^3 m-3"
           units_tp(2) = "kg m-2 s-1"
           units_tp(3) = "kg m-2 s-1"
           units_tp(4) = "1"
           units_tp(5) = "K"
           units_tp(6) = "kg m-2"

           call LIS_forwardEstimate_with_ANN(n, gid, inputs_tp, &
                units_tp, sm_out)
           obs_pred(gid,m) = obs_pred(gid,m) + sm_out
           count1(gid,m) = count1(gid,m) + 1
        enddo
     enddo
     
     do i=1,LIS_rc%ngrid(n)
        do m=1,LIS_rc%nensem(n)
           obs_pred(i,m) = obs_pred(i,m)/(count1(i,m))
        enddo
     enddo
  endif
end subroutine noah33_getsmpred

